% Vaccines - 2019-01-31

clear all
clc

global B_backup;


%% ****************************  loading data  ****************************

EstimOpt.DataFile = ('vaccines_data_01.mat');
DATA = load(EstimOpt.DataFile);
EstimOpt.DesignFile = ('vaccines_design_01.mat');
DESIGN = load(EstimOpt.DesignFile);

%% ****************************  data transformations  ****************************

% convert short to long:

NAlt = 2;
NCT = 12;

RID = unique(DATA.PID);

DATA.Alt = repmat([1:NAlt]',[NCT*size(RID,1),1]);
DATA.CT = repmat([1:NCT],[NAlt,1]);
DATA.CT = repmat(DATA.CT(:),[size(RID,1),1]);

DATA.RID = repmat(RID,[1,NCT*NAlt])';
DATA.RID = DATA.RID(:);

DATA.ASC = NaN(NAlt*NCT*size(RID,1),1);
DATA.Efficacy = NaN(NAlt*NCT*size(RID,1),1);
DATA.Frequency = NaN(NAlt*NCT*size(RID,1),1);
DATA.Tagging = NaN(NAlt*NCT*size(RID,1),1);
DATA.Authority = NaN(NAlt*NCT*size(RID,1),1);
DATA.Cost = NaN(NAlt*NCT*size(RID,1),1);

DATA.BlockLong = repmat(DATA.Block,[1,NCT*NAlt])';
DATA.BlockLong = DATA.BlockLong(:);

DATA.CARD7(DATA.CARD7 == 2) = 1;

for i = 1 : NCT    
    % alt 1:
    DATA.ASC(DATA.Alt == 1) = 0;
    DATA.Efficacy(DATA.Alt == 1) = 0; % baseline - efficacy 50%
    DATA.Frequency(DATA.Alt == 1) = 1; % baseline - frequency twice a year
    DATA.Tagging(DATA.Alt == 1) = 0; % baseline - no tagging
    DATA.Authority(DATA.Alt == 1) = 3; % baseline - government vet
    DATA.Cost(DATA.Alt == 1) = 0;
    
    % alt 2:
    DATA.ASC(DATA.Alt == 2) = 1;
    for k = 1:6
        DATA.Efficacy(DATA.CT == i & DATA.Alt == 2 & DATA.BlockLong == k) = DESIGN.Efficacy(DESIGN.Block==k & DESIGN.Choicesituation==i);
        DATA.Frequency(DATA.CT == i & DATA.Alt == 2 & DATA.BlockLong == k) = DESIGN.Frequency(DESIGN.Block==k & DESIGN.Choicesituation==i);
        DATA.Tagging(DATA.CT == i & DATA.Alt == 2 & DATA.BlockLong == k) = DESIGN.Tagging(DESIGN.Block==k & DESIGN.Choicesituation==i);
        DATA.Authority(DATA.CT == i & DATA.Alt == 2 & DATA.BlockLong == k) = DESIGN.Authority(DESIGN.Block==k & DESIGN.Choicesituation==i);
        DATA.Cost(DATA.CT == i & DATA.Alt == 2 & DATA.BlockLong == k) = DESIGN.Cost1(DESIGN.Block==k & DESIGN.Choicesituation==i);
    end
    
    % ans
    for j = 1 : NAlt
        DATA.ANS((DATA.CT == i) & (DATA.Alt == j),:) = eval(['DATA.CARD' num2str(i)]);        
    end
end

DATA.ANS = DATA.ANS == (DATA.Alt-1);

clear NAlt NCT RID i j k

DATA.Frequency(DATA.Alt==1) = 0;

% o   (a) how many cases of MCF the farmer has experienced in say the past x years, this is question D6 in the survey/data set(we suggest using data from 2017 and 2016)
DATA.mcfcases2016(isnan(DATA.mcfcases2016)) = 0;
DATA.mcfcases2017(isnan(DATA.mcfcases2017)) = 0;
DATA.mcfcases_2016_2017 = ReplicateRows(DATA.mcfcases2016 + DATA.mcfcases2017, 12*2);
DATA.log_mcfcases_2016_2017 = log(DATA.mcfcases_2016_2017 + 1);

% o   (c ) The number of cattle a household owns (might be correlated with income); 
DATA.cattleowned = ReplicateRows(DATA.cattleowned, 12*2);
DATA.log_cattleowned = log(DATA.cattleowned);

% o   (d) How many cattle the farmer moved away from the high risk area to reduce his exposure to risk this is D9
DATA.numbercattlemovingaway(DATA.numbercattlemovingaway==2) = 0.5;
DATA.numbercattlemovingaway(DATA.numbercattlemovingaway==3) = 0.25;
DATA.numbercattlemovingaway(DATA.numbercattlemovingaway==4) = 0;
DATA.numbercattlemovingaway = ReplicateRows(DATA.numbercattlemovingaway,12*2);



%% ****************************  model specification  ****************************

DATA.Y = DATA.ANS;

DATA.X = [DATA.Alt==1, DATA.Efficacy, DATA.Frequency, DATA.Tagging == 1, DATA.Authority == 1, DATA.Authority == 3, DATA.Cost/10000];   
EstimOpt.NamesA = {'Status quo';'Efficacy (%)';'Frequency (/y)';'Tagging'; 'Private vet (vs. NGO)'; 'Government vet (vs. NGO)'; 'Cost (10000 TZS)'};

DATA.Xm = [DATA.log_mcfcases_2016_2017,norm_var(DATA.cattleowned), DATA.numbercattlemovingaway];
EstimOpt.NamesM = {'log MCF cases','cattle owned','% cattle moved away'};


%% ****************************  specifying input ****************************


DATA.filter = true(size(DATA.RID));
% DATA.filter = DATA.BlockLong < 4;

INPUT.Y = DATA.Y(DATA.filter); 
INPUT.Xa = DATA.X(DATA.filter,:); 

% INPUT.Xm = norm_var(DATA.Xm(DATA.filter,:));

% EstimOpt.NLTVariables = 7; 
% EstimOpt.NLTType = 1;

EstimOpt.ProjectName = 'baseline';
% EstimOpt.ProjectName = 'interactions';
% EstimOpt.ProjectName = 'non_linear';


%% ****************************  sample characteristics ****************************


EstimOpt.NCT = 12; % Number of choice tasks per person 
EstimOpt.NAlt = 2; % Number of alternatives
EstimOpt.NP = length(INPUT.Y)/EstimOpt.NCT/EstimOpt.NAlt; % numel(unique(DATA.nr))


%% **************************** estimation and optimization options ****************************


[INPUT, Results, EstimOpt, OptimOpt] = DataCleanDCE(INPUT,EstimOpt);

EstimOpt.NRep = 1e4; % number of draws for numerical simulation


%% ****************************     MNL     ****************************

% baseline model: -1478.6299
B_backup = [0.389651881997879;2.67616677593822;0.00356462801168282;1.16197305168697;0.308302841915152;0.122112265995968;-0.822314803110017];

% interactions model: -1450.5876
% B_backup = [0.457714769747072;2.85489502893564;-0.00751585642796123;1.14333881415845;0.277673420979245;0.135365912688739;-0.809907073663560;0.115702948856400;-0.0668052787910879;0.0760760024282014;0.291949523220636;0.132142547210367;0.132794391083107;-0.0796360028130148;-0.370726237171309;-0.512477529259205;-0.131620801688197;-0.0299542795898685;-0.0488045598789568;-0.268975248010123;0.141593163634472;-0.0846110676636389;-1.23357927598626;0.0519712331910476;0.389255911464686;0.0783292442971173;-0.0911604183913412;0.167500361412431];

% non-linear cost model: -1477.1780
% B_backup = [0.725844748444959;2.58055608387386;0.00886179928791979;1.14187813319828;0.361503582831345;0.118609978478882;0.637831817672858;1.62671511628238];


OptimOpt.Algorithm = 'trust-region'; % 'quasi-newton';
OptimOpt.Hessian = 'user-supplied'; % 'off'

Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);

OptimOpt.Algorithm = 'quasi-newton';
OptimOpt.Hessian = 'off';

Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);


%% simulate WTP (for baseline model)


sim1 = 1e4;
rng(EstimOpt.Seed1);
b_mtx = Results.MNL.bhat(1:size(INPUT.Xa,2));
hess_mtx = Results.MNL.ihess;
b_mtx_sim = mvnrnd(b_mtx,hess_mtx,sim1);

WTP = -[b_mtx(1:end-1);-b_mtx(1)+0.9*b_mtx(2)+b_mtx(3)+b_mtx(4)+b_mtx(5)]./b_mtx(end);
WTP_sim = -([b_mtx_sim(:,1:end-1),-b_mtx_sim(:,1)+0.9*b_mtx_sim(:,2)+b_mtx_sim(:,3)+b_mtx_sim(:,4)+b_mtx_sim(:,5)]./b_mtx_sim(:,end))';

WTP(:,2) = std(WTP_sim,[],2);
WTP(:,4) = quantile(WTP_sim,0.025,2);
WTP(:,5) = quantile(WTP_sim,0.975,2);
WTP = WTP*10000;
WTP(2,:) = WTP(2,:)/100;
WTP(:,3) = pv(WTP(:,1),WTP(:,2));


% without uncertainty


max_price = 6;

XXa = [1 0   0  0 0 0 0;...
       0 0.9 1  1 1 0 0]; 
delta = 0;
delta(2,1:size(0:2.5/1e4:max_price,2)) = 0:2.5/1e4:max_price;
delta = reshape(delta,[2,1,size(delta,2)]);
delta = [zeros(2,6,size(delta,3)),delta];
XXa = repmat(XXa,[1,1,size(delta,3)]);
% XXa = reshape(XXa,[size(delta,1)/2,7,2]);
% XXa = permute(XXa,[3,2,1]);
XXa = XXa + delta;

b_mtx = Results.MNL.bhat(1:size(XXa,2));

p0 = zeros(1,size(XXa,3));


if isfield(EstimOpt,'NLTVariables')
    XXa0 = XXa;
    
    bt = Results.MNL.bhat(end);
    IndTransNon0 = (abs(bt) > eps)';
    Xt = XXa0(:,end,:);
    if EstimOpt.NLTType == 1 % BC
        Xt(:,IndTransNon0,:) = -(Xt(:,IndTransNon0 == 1,:).^(bt(IndTransNon0 == 1,:)') - 1)./bt(IndTransNon0 == 1,:)';
        Xt(:,~IndTransNon0,:) = -log(Xt(:,IndTransNon0 == 0,:));
%     elseif EstimOpt.NLTType == 2 % YJ
%         IndXtNon0 = (Xt >= 0);
%         IndXtCase1 = IndXtNon0 & IndTransNon0; % X >= 0, lam ~= 0
%         IndXtCase2 = IndXtNon0 & ~IndTransNon0; % X >= 0, lam = 0
%         IndTransNon2 = (abs(bt - 2) > eps)';
%         IndXtCase3 = ~IndXtNon0 & IndTransNon2;  % X < 0, lam ~= 2
%         IndXtCase4 = ~IndXtNon0 & ~IndTransNon2; % X < 0, lam = 2
%         bt_tmp = bt(:,ones(size(Xa,1),1))';
%         Xt(IndXtCase1) = ((Xt(IndXtCase1) + 1).^bt_tmp(IndXtCase1) - 1)./bt_tmp(IndXtCase1);
%         Xt(IndXtCase2) = log(Xt(IndXtCase2) + 1);
%         Xt(IndXtCase3) = -((-Xt(IndXtCase3) + 1).^(2 - bt_tmp(IndXtCase3)) - 1)./(2 - bt_tmp(IndXtCase3));
%         Xt(IndXtCase4) = -log(-Xt(IndXtCase4) + 1);
    end
    XXa(:,end,:) = Xt;
end

for i = 1:size(XXa,3)    
        U = XXa(:,:,i)*b_mtx;
        U = exp(U - max(U,[],1)); % rescale utility to avoid exploding
        U_sum = sum(U,1);
        p0(i) = U(2)./U_sum;    
end


%% with uncertainty:

b_mtx = Results.MNL.bhat;
hess_mtx = Results.MNL.ihess;

sim1 = 1e4;
rng(EstimOpt.Seed1);

b_mtx_sim = mvnrnd(b_mtx,hess_mtx,sim1);

p0_sim_k = zeros(size(delta,3),1);
p0_sim = zeros(size(delta,3),sim1);

tic
for k = 1:sim1    
    
    b_mtx = b_mtx_sim(k,1:size(XXa,2))';
    
    if isfield(EstimOpt,'NLTVariables')
        bt = b_mtx_sim(k,end);
        IndTransNon0 = (abs(bt) > eps)';
        Xt = XXa0(:,end,:);
        if EstimOpt.NLTType == 1 % BC
            Xt(:,IndTransNon0,:) = -(Xt(:,IndTransNon0 == 1,:).^(bt(IndTransNon0 == 1,:)') - 1)./bt(IndTransNon0 == 1,:)';
            Xt(:,~IndTransNon0,:) = -log(Xt(:,IndTransNon0 == 0,:));
            %     elseif EstimOpt.NLTType == 2 % YJ
            %         IndXtNon0 = (Xt >= 0);
            %         IndXtCase1 = IndXtNon0 & IndTransNon0; % X >= 0, lam ~= 0
            %         IndXtCase2 = IndXtNon0 & ~IndTransNon0; % X >= 0, lam = 0
            %         IndTransNon2 = (abs(bt - 2) > eps)';
            %         IndXtCase3 = ~IndXtNon0 & IndTransNon2;  % X < 0, lam ~= 2
            %         IndXtCase4 = ~IndXtNon0 & ~IndTransNon2; % X < 0, lam = 2
            %         bt_tmp = bt(:,ones(size(Xa,1),1))';
            %         Xt(IndXtCase1) = ((Xt(IndXtCase1) + 1).^bt_tmp(IndXtCase1) - 1)./bt_tmp(IndXtCase1);
            %         Xt(IndXtCase2) = log(Xt(IndXtCase2) + 1);
            %         Xt(IndXtCase3) = -((-Xt(IndXtCase3) + 1).^(2 - bt_tmp(IndXtCase3)) - 1)./(2 - bt_tmp(IndXtCase3));
            %         Xt(IndXtCase4) = -log(-Xt(IndXtCase4) + 1);
        end
        XXa(:,end,:) = Xt;
    end
    
    for i = 1:size(XXa,3)
        U = XXa(:,:,i)*b_mtx;
        U = exp(U - max(U,[],1)); % rescale utility to avoid exploding
        U_sum = sum(U,1);
        p0_sim_k(i) = U(2)./U_sum;
    end

    p0_sim(:,k) = p0_sim_k;
    
end
toc        
    
p0_l = quantile(p0_sim,0.025,2);
p0_u = quantile(p0_sim,0.975,2);
%     
%    
% figure
% hold on
% Pplot(1) = plot(0:2.5/1e4:max_price,p0,'b-');
% Pplot(2) = plot(0:2.5/1e4:max_price,p0_l,'r--');
% Pplot(3) = plot(0:2.5/1e4:max_price,p0_u,'r--');
% set(Pplot(1),'linewidth',2);
% xlabel('Cost [10,000 TZS]'); 
% ylabel('Probability');
% set(gcf,'Color', 'w')
% legend([Pplot(1),Pplot(2)],'Probability of acceptance','95% confidence interval','Location', 'northeast')
% set(gca,'Position',[.10 .12 .85 .82]);
% saveas(gcf,'Probability plot','png')

