% Vaccines - 2019-01-31

clear all
clc

global B_backup;


%% ****************************  loading data  ****************************

EstimOpt.DataFile = ('vaccines_data_01.mat');
DATA = load(EstimOpt.DataFile);
EstimOpt.DesignFile = ('vaccines_design_01.mat');
DESIGN = load(EstimOpt.DesignFile);

%% ****************************  data transformations  ****************************

% convert short to long:

NAlt = 2;
NCT = 12;

RID = unique(DATA.PID);

DATA.Alt = repmat([1:NAlt]',[NCT*size(RID,1),1]);
DATA.CT = repmat([1:NCT],[NAlt,1]);
DATA.CT = repmat(DATA.CT(:),[size(RID,1),1]);

DATA.RID = repmat(RID,[1,NCT*NAlt])';
DATA.RID = DATA.RID(:);

DATA.ASC = NaN(NAlt*NCT*size(RID,1),1);
DATA.Efficacy = NaN(NAlt*NCT*size(RID,1),1);
DATA.Frequency = NaN(NAlt*NCT*size(RID,1),1);
DATA.Tagging = NaN(NAlt*NCT*size(RID,1),1);
DATA.Authority = NaN(NAlt*NCT*size(RID,1),1);
DATA.Cost = NaN(NAlt*NCT*size(RID,1),1);

DATA.BlockLong = repmat(DATA.Block,[1,NCT*NAlt])';
DATA.BlockLong = DATA.BlockLong(:);

DATA.CARD7(DATA.CARD7 == 2) = 1;

for i = 1 : NCT    
    % alt 1:
    DATA.ASC(DATA.Alt == 1) = 0;
    DATA.Efficacy(DATA.Alt == 1) = 0; % baseline - efficacy 50%
    DATA.Frequency(DATA.Alt == 1) = 1; % baseline - frequency twice a year
    DATA.Tagging(DATA.Alt == 1) = 0; % baseline - no tagging
    DATA.Authority(DATA.Alt == 1) = 3; % baseline - government vet
    DATA.Cost(DATA.Alt == 1) = 0;
    
    % alt 2:
    DATA.ASC(DATA.Alt == 2) = 1;
    for k = 1:6
        DATA.Efficacy(DATA.CT == i & DATA.Alt == 2 & DATA.BlockLong == k) = DESIGN.Efficacy(DESIGN.Block==k & DESIGN.Choicesituation==i);
        DATA.Frequency(DATA.CT == i & DATA.Alt == 2 & DATA.BlockLong == k) = DESIGN.Frequency(DESIGN.Block==k & DESIGN.Choicesituation==i);
        DATA.Tagging(DATA.CT == i & DATA.Alt == 2 & DATA.BlockLong == k) = DESIGN.Tagging(DESIGN.Block==k & DESIGN.Choicesituation==i);
        DATA.Authority(DATA.CT == i & DATA.Alt == 2 & DATA.BlockLong == k) = DESIGN.Authority(DESIGN.Block==k & DESIGN.Choicesituation==i);
        DATA.Cost(DATA.CT == i & DATA.Alt == 2 & DATA.BlockLong == k) = DESIGN.Cost1(DESIGN.Block==k & DESIGN.Choicesituation==i);
    end
    
    % ans
    for j = 1 : NAlt
        DATA.ANS((DATA.CT == i) & (DATA.Alt == j),:) = eval(['DATA.CARD' num2str(i)]);        
    end
end

DATA.ANS = DATA.ANS == (DATA.Alt-1);

clear NAlt NCT RID i j k

DATA.Frequency(DATA.Alt==1) = 0;

% o   (a) how many cases of MCF the farmer has experienced in say the past x years, this is question D6 in the survey/data set(we suggest using data from 2017 and 2016)
DATA.mcfcases2016(isnan(DATA.mcfcases2016)) = 0;
DATA.mcfcases2017(isnan(DATA.mcfcases2017)) = 0;
DATA.mcfcases_2016_2017 = ReplicateRows(DATA.mcfcases2016 + DATA.mcfcases2017, 12*2);
DATA.log_mcfcases_2016_2017 = log(DATA.mcfcases_2016_2017 + 1);

% o   (b) household income, this is question E
DATA.householdincome = ReplicateRows(DATA.householdincome/10000, 12*2);
DATA.log_householdincome = log(DATA.householdincome);


%% ****************************  model specification  ****************************


DATA.Y = DATA.ANS;

DATA.X = [DATA.Alt==1, DATA.Efficacy, DATA.Frequency, DATA.Tagging == 1, DATA.Authority == 1, DATA.Authority == 3, -DATA.Cost/10000];   
EstimOpt.NamesA = {'Status quo';'Efficacy (%)';'Frequency (/y)';'Tagging'; 'Private vet (vs. NGO)'; 'Government vet (vs. NGO)'; '-Cost (10000 TZS)'};

DATA.Xc = [DATA.log_mcfcases_2016_2017,DATA.log_householdincome];
EstimOpt.NamesC = {'log MCF cases','log HH income'};


%% ****************************  specifying input ****************************


DATA.filter = true(size(DATA.RID));

INPUT.Y = DATA.Y(DATA.filter); 
INPUT.Xa = DATA.X(DATA.filter,:); 

INPUT.Xc = norm_var(DATA.Xc(DATA.filter,:));


%% ****************************  sample characteristics ****************************


EstimOpt.NCT = 12; % Number of choice tasks per person 
EstimOpt.NAlt = 2; % Number of alternatives
EstimOpt.NP = length(INPUT.Y)/EstimOpt.NCT/EstimOpt.NAlt; % numel(unique(DATA.nr))


%% **************************** estimation and optimization options ****************************


[INPUT, Results, EstimOpt, OptimOpt] = DataCleanDCE(INPUT,EstimOpt);

EstimOpt.NRep = 1e4; % number of draws for numerical simulation


%% ****************************     MNL     ****************************

EstimOpt.WTP_space = 1;

B_backup = [0.473847604824035;3.25443108772373;0.00433485488041649;1.41305128726397;0.374920790870707;0.148498235290620;0.822314816815958];

% OptimOpt.Algorithm = 'trust-region'; % 'quasi-newton';
% OptimOpt.Hessian = 'user-supplied'; % 'off'
% 
% Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);

OptimOpt.Algorithm = 'quasi-newton';
OptimOpt.Hessian = 'off';

Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);


%% ****************************     LC     ****************************


% 3 class model fits best:
EstimOpt.NClass = 3;

% B_backup = [0.967624312699853;3.10045225576079;0.136577866956292;1.98110287044647;0.170923361229876;0.363224896666455;2.65275747074685;0.575933466539692;3.98655636440849;-0.0884483369301993;1.33783398584507;0.119418930930596;-0.0784907108394246;0.642333182828754;-1.47956803722173;0.362747039201642;0.162000110316666;-1.20822828154211;2.15524372875182;-0.0265409501969612;5.52188389917011;1.45126409119641;2.59534713636452];
% B_backup = [B_backup(1:end-1); zeros(size(INPUT.Xc,2),1); B_backup(end);zeros(size(INPUT.Xc,2),1)];
B_backup = [1.11022270300693;3.30761559997034;0.133824535284585;1.73607746332146;0.502555077985710;0.395425089512112;1.49236654883500;0.499312133494915;4.90249619263455;-0.249598562760442;1.53240271704131;-0.195165350544565;-0.298714830112536;0.470701721189853;-1.94258566961226;0.0137580715435097;0.111299792211498;-1.44269319430674;1.80304297971541;-0.114070277892323;4.12286887469826;3.79430772260455;1.68266855415293;2.49745834886677;4.01327168025824;1.04545775541270;2.85338020102078];

% OptimOpt.Algorithm = 'trust-region'; % 'quasi-newton';
% OptimOpt.Hessian = 'user-supplied'; % 'off'
% 
% Results.LC = LC(INPUT,Results,EstimOpt,OptimOpt);

OptimOpt.Algorithm = 'quasi-newton';
OptimOpt.Hessian = 'off';

Results.LC = LC(INPUT,Results,EstimOpt,OptimOpt);


%% simulate WTP (for LC)


sim1 = 1e4;
rng(EstimOpt.Seed1);

b_mtx = Results.LC.bhat;
hess_mtx = Results.LC.ihess;
b_mtx_sim = mvnrnd(b_mtx,hess_mtx,sim1);

WTP = reshape(b_mtx(1:21),[7,3]);
WTP(end,:) = [];
WTP(end+1,:) = [-WTP(1,:) + 0.9*WTP(2,:) + WTP(3,:) + WTP(4,:) + WTP(5,:)];
WTP(end+1,1) = [WTP(end,1).*Results.LC.PClass(1,1)/100 + WTP(end,2).*Results.LC.PClass(1,5)/100 + WTP(end,3).*Results.LC.PClass(1,9)/100];

WTP_sim = reshape(b_mtx_sim(:,1:21),[sim1,7,3]);
WTP_sim(:,end,:) = [];
WTP_sim(:,end+1,:) = [-WTP_sim(:,1,:) + 0.9*WTP_sim(:,2,:) + WTP_sim(:,3,:) + WTP_sim(:,4,:) + WTP_sim(:,5,:)];
WTP_sim(:,end+1,1) = [WTP_sim(:,end,1).*Results.LC.PClass(1,1)/100 + WTP_sim(:,end,2).*Results.LC.PClass(1,5)/100 + WTP_sim(:,end,3).*Results.LC.PClass(1,9)/100];

WTP_se = squeeze(std(WTP_sim,[],1));
WTP_q025 = squeeze(quantile(WTP_sim,0.025,1));
WTP_q975 = squeeze(quantile(WTP_sim,0.975,1));
WTP_pv = pv(WTP,WTP_se);



%% plot acceptance probabilities (for LC)

max_price = 6;

XXa = [1 0   0  0 0 0 0;...
       0 0.9 1  1 1 0 0]; 
   
delta = 0;
delta(2,1:size(0:2.5/1e4:max_price,2)) = 0:2.5/1e4:max_price;
delta = reshape(delta,[2,1,size(delta,2)]);
delta = [zeros(2,6,size(delta,3)),delta];
XXa = repmat(XXa,[1,1,size(delta,3)]);
XXa = XXa - delta;

b_mtx = Results.LC.bhat(1:end);
p0 = zeros(size(XXa,3),EstimOpt.NClass);

WTP_att = (EstimOpt.NVarA - EstimOpt.WTP_space)/EstimOpt.WTP_space;
EstimOpt.WTP_matrix = EstimOpt.NVarA - EstimOpt.WTP_space + kron(1:EstimOpt.WTP_space,ones(1,WTP_att));

Bclass = reshape(b_mtx(1:EstimOpt.NClass*EstimOpt.NVarA),[EstimOpt.NVarA,EstimOpt.NClass]);
if EstimOpt.WTP_space > 0    
    Bclass(1:end-EstimOpt.WTP_space,:) = Bclass(1:end-EstimOpt.WTP_space,:).*Bclass(EstimOpt.WTP_matrix,:);
end

for i = 1:size(XXa,3)
    U = XXa(:,:,i)*Bclass; % NAlt x NClass
    U = exp(U-max(U));
    p0(i,:) =  U(2,:)./sum(U,1); % NAlt x NCT*NP*NClass
end

p0(:,4) = Results.LC.PClass(1,1).*p0(:,1)./100 + Results.LC.PClass(1,5).*p0(:,2)./100 + Results.LC.PClass(1,9).*p0(:,3)./100;
p0(:,5) = delta(2,7,:);

figure
hold on
Pplot(1) = plot(0:2.5/1e4:max_price,p0(:,1),'b--');
Pplot(2) = plot(0:2.5/1e4:max_price,p0(:,2),'g:');
Pplot(3) = plot(0:2.5/1e4:max_price,p0(:,3),'c-.');
Pplot(4) = plot(0:2.5/1e4:max_price,p0(:,4),'r-');
set(Pplot(1),'linewidth',1.2);
set(Pplot(2),'linewidth',1.2);
set(Pplot(3),'linewidth',1.2);
set(Pplot(4),'linewidth',2);
xlabel('Cost [10,000 TZS]'); 
ylabel('Probability of acceptance');
set(gcf,'Color', 'w')
legend([Pplot(1),Pplot(2),Pplot(3),Pplot(4)],'Class 1','Class 2','Class 3','Overall','Location', 'northeast')
set(gca,'Position',[.10 .12 .85 .82]);
saveas(gcf,'Probability plot','png')



